#!/usr/bin/perl

# This Perl script synchronizes groups in an LDAP directory with contact
# objects from Nagios: It is given a elementary configuration file defining
# which user/password and server address to use to contact the webservices API
# of op5 Monitor. Additionally, it reads the authentication configuration file
# of op5lib (default location: /etc/op5/auth.yml) to get access to the LDAP
# server.  The script will run through all LDAP groups that are relevant to op5
# Monitor's Login configuration and check if there is a corresponding
# contactgroup in op5 Monitor's configuration. If this is the case, it will
# then go through all users in this specific LDAP group and - if the
# corresponding user is not existing as contact object - create it. Then the
# contactgroup assignments are synchronized so that the LDAP settings are fully
# represented in the local op5 Monitor's contact/contactgroups structure.  This
# way, it is possible to assign the contactgroup objects to hosts/services
# accordingly to make use of multi-user configuration, based on an LDAP backend
# and allowing users with limited view permissions.

#Copyright © 2013 Christian Anton. All Rights Reserved.

#This program is free software; you can redistribute it and/or modify it under
#the terms of the GNU General Public License as published by the Free Software
#Foundation; either version 2 of the License, or (at your option) any later
#version.

#This program is distributed in the hope that it will be useful, but WITHOUT
#ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#You should have received a copy of the GNU General Public License along with
#this program; if not, write to the Free Software Foundation, Inc., 59 Temple
#Place, Suite 330, Boston, MA 02111-1307 USA

##### changelog
# 2014-02-05 v0.1.3 <mail@christiananton.de> initial version
# 2014-02-13 v0.2.0 <mail@christiananton.de> Some new features and bug fixes
#            - added function explanation comment
#            - added proper help function
#            - undefined port number now defaults to 389 (if nothing configured)
#            - anonymous LDAP bind is now possible
#            - bug fixed: script didn't work with memberkey_is_dn set to false
#            - made check_monitor_has_contact function more effective and fast
#            - bug fixed: empty LDAP groups are now handled properly
# 2014-06-04 v0.3.2 <mail@christiananton.de> fixed a bug with UPN names in LDAP
# 2014-06-12 v0.4.0 <mail@christiananton.de> added notification to op5 Monitor
#            using a passive service check object. Also added the creation of 
#            this service check by the script itself (--add-notification-service-check)
# 2014-06-12 v0.4.1 <mail@christiananton.de> RPM packaging error
# 2014-06-12 v0.4.2 <mail@christiananton.de> missing commenting some lines
# 2014-06-16 v0.4.3 <mail@christiananton.de> more testing for existing configuration
#                                            for adding the notification service check
# 2015-11-20 v0.4.4 <mail@christiananton.de> Added feature to make sending notifications
#                                            possible for the automatically created contact
#                                            objects
# 2015-12-01 v0.5.0 <mail@christiananton.de> Added support for nested groups inside of
#                                            matched LDAP groups.

use strict;
use warnings;
use Net::LDAP; # not installed in APS by default, but available: yum install perl-LDAP
use Sys::Syslog qw(:standard :macros); # default module contained in perl package on CentOS
use Data::Dumper; # default module contained in perl package on CentOS
use LWP; # installed by default on APS, contained in package perl-libwww-perl
use JSON::PP; # not installed in APS by default, but available: yum install perl-JSON
use URI::Escape; # installed by default on APS, contained in package perl-URI
use Getopt::Long; # default module contained in perl package on CentOS
use YAML qw(LoadFile); # not installed in APS by default, but available: yum install perl-YAML

##### global configuration variables #####
my $o_config_file = "/opt/op5-ldap-helper/op5-ldap-helper.conf.yml";
my $authyml;
my $o_help;
my $o_pretend;
my $o_debug;
my $o_nosave;
my $o_consoleout;
my $o_authyml = '/etc/op5/auth.yml';
my $o_add_notification_service_check;
my $o_nonotify;
my $config;
my $ldap;
my @ldap_group_members_shortnames;
my $affected_ldap_groups = 0;
my $total_affected_ldap_users = 0;
my $added_contacts = 0;
my $updated_contactgroups = 0;
my $deleted_contacts = 0;

check_options();
$config = LoadFile($o_config_file);

#### SUBs ####
sub load_authyml {
  my $ldap_settings = LoadFile($o_authyml);

  return $ldap_settings->{$config->{authyml_connection_name}};
}

sub print_usage {
    print "Usage: $0 [-h|--help] [-p|--pretend] [--nonotify] [-d|--debug] [-n|--nosave] [-C|--consoleout]\n";
    print "  [--add-notification-service-check]\n";
    print "  [-c|--config <op5-ldap-helper.conf.yml>]\n";
    print "  [-a|--authyml <auth.yml>]\n\n";
}

sub print_help {
  print_usage();
  print <<"EOT";
-h, --help
    print this help message
-d, --debug
    turn debug logging on
-C, --consoleout
    output all messages to the console and not only to syslog
-c, --config <FILE>
    set configuration file. Default is /opt/op5-ldap-helper/op5-ldap-helper.conf.yml
-a, --authyml <FILE>
    set authentication configuration file. Default is /etc/op5/auth.yml
--add-notification-service-check
    add the service check object to op5 Monitor that will be used to show the status of the
    last run of this script, containing also some statistical values about how
    many contact objects have been created etc.
--nonotify
    Do not notify op5 Monitor about the successful (or unsuccessful) execution of this script.
    This notification is usually done submitting a passive service check result to the service
    check object configured with the "service_check_description" configuration setting, assigned
    to the host object configured by the "host_object_name" configuration variable.
    For this to work, the configuration setting "notify_service_check" must be set to "true".
-n, --nosave
    Do not save changes in op5 Monitor API after this script finishes.
    This allows you to check in the graphical configuration manager which changes had
    been issued by this script before writing configuration to op5 Monitor
    (this is good for testing)
-p, --pretend
    Do not actually write anything to the op5 Monitor API at all. Instead, debugging
    messages showing what would actually be done (dumping JSON code) will be displayed
EOT
  exit 0;
}

sub check_options {
  Getopt::Long::Configure("bundling");
  GetOptions(
    'h'   => \$o_help,    'help'    => \$o_help,
    'p'   => \$o_pretend, 'pretend' => \$o_pretend,
    'd'   => \$o_debug,   'debug' => \$o_debug,
    'C'   => \$o_consoleout, 'consoleout' => \$o_consoleout,
    'c:s' => \$o_config_file, 'config:s' => \$o_config_file,
    'n'   => \$o_nosave,  'nosave'  => \$o_nosave,
                          'nonotify' => \$o_nonotify,
    'a:s' => \$o_authyml, 'authyml:s' => \$o_authyml,
    'add-notification-service-check' => \$o_add_notification_service_check
  );

  if (defined $o_help) { print_help; }
}

sub ldap_con {
	my $host = $authyml->{server};
  my $port = $authyml->{port};
	my $ldap;

  # default port if not defined in op5 Monitor config GUI
  if ($port eq "false" or $port eq "") { $port = 389; }

	do_msg("info","initiating connection to ldap srv $host with port $port");
	
  $ldap = Net::LDAP->new (
    "$host",
    port => $port,
    scheme => 'ldap',
    onerror => 'warn',
    timeout => 3
    ) ;

	return $ldap;
}

sub ldap_bind {
	my $ldap = shift;
	my $binddn = shift;
	my $password = shift;
	my $mesg;

  if ($binddn eq "false" or $binddn eq "") {
    $mesg = $ldap->bind;
  } else {
    $mesg = $ldap->bind ( 
      "$binddn",
      password => "$password",
      );
  }

  if ($mesg->code != 0) {
    end_script(2, "LDAP bind-dn could not authenticate");
  } else {
    do_msg("info","bind-dn authenticated successfully");
  }

	return $mesg;
}

sub do_msg {
	# can be "info", "warning", "error"
	my $level = shift; 
	my $msg = shift;

	if ($level eq "info") {
		if (defined($o_debug)) {
			print "$msg\n" if defined($o_consoleout);
      syslog($level, $msg);
		}
	} else {
		print "$msg\n" if defined($o_consoleout);
    syslog($level, $msg);
	}

}

sub end_script {
  my $exitcode = shift;
  my $msg = shift;

  my $statusmap = {
    0 => 'OK: ',
    1 => 'WARNING: ',
    2 => 'CRITICAL: ',
    3 => 'UNKNOWN: '
  };

  my $loglevelmap = {
    0 => 'info',
    1 => 'warning',
    2 => 'LOG_ERR',
    3 => 'LOG_ERR'
  };

  my ($shortmsg, $perfdata) = split(' \| ', $msg);
  do_msg($loglevelmap->{$exitcode}, $shortmsg);

  if (!$o_nonotify and $config->{notify_service_check} and $config->{notify_service_check}->{notify_service_check} and $config->{notify_service_check}->{notify_service_check} eq "true") {

    do_msg("info","Notifying op5 Monitor service check \"" . $config->{notify_service_check}->{service_check_description} . "\" on host \"" . $config->{notify_service_check}->{host_object_name} . "\""); 

    my %service_check_result = (
      'host_name' => $config->{notify_service_check}->{host_object_name},
      'service_description' => $config->{notify_service_check}->{service_check_description},
      'status_code' => $exitcode,
      'plugin_output' => "Script " . $statusmap->{$exitcode} . $msg
    );

    post_op5_api_url('https://' . $config->{op5api}->{server} . '/api/command/PROCESS_SERVICE_CHECK_RESULT', encode_json( \%service_check_result ) );

  } else {
    do_msg("info","skipping notification of op5 Monitor service object"); 
  }

  closelog;
  exit $exitcode;
}

sub array_usort {
	my %all;
	$all{$_}=0 for @_;
	return (keys %all);
} 

# op5 API related functions
sub prepare_http_request {
  my $op5_api_authentication_realm;

  my $http_browser = LWP::UserAgent->new;
  $http_browser->agent('op5-ldap-helper/0.5.0');

  # check if a non-standard authentication realm is set in config and use it, otherwise
  # use the default that op5 Monitor should be set to
  if ($config->{op5api}->{authentication_realm}) {
    $op5_api_authentication_realm = $config->{op5api}->{authentication_realm};
  } else {
    $op5_api_authentication_realm = 'op5 Monitor API Access';
  }

  $http_browser->credentials(
    $config->{op5api}->{server} . ':443',
    $op5_api_authentication_realm,
    $config->{op5api}->{user},
    $config->{op5api}->{password}
  );

  if ($http_browser->can('ssl_opts')) {
    if ($config->{op5api}->{ssl_verify_hostname} and $config->{op5api}->{ssl_verify_hostname} eq "true") {
      $http_browser->ssl_opts(
        verify_hostname => 1
      );
    } else {
      $http_browser->ssl_opts(
        verify_hostname => 0
      );
    }
  }
  return $http_browser;
}

sub get_op5_api_url {
  my $url = shift;
  my $http_browser = prepare_http_request();
  my $response = $http_browser->get($url);

  if ($response->is_success) {
    return $response->content;
  } else {
    my $content = decode_json($response->content);
    end_script(2, "Error issuing GET request on API: " . $response->status_line . " - " . $content->{full_error});
  }
}

sub post_op5_api_url {
  my $url = shift;
  my $json = shift;

  if ($o_pretend) {
    do_msg("info", "pretending op5 API POST request on URL: " . $url);
    do_msg("info", $json);
    return 500;
  }

  my $http_browser = prepare_http_request();

  my $req = HTTP::Request->new(POST => $url);
  $req->content_type('application/json');
  $req->content($json);

  my $res = $http_browser->request($req);
  if ($res->is_success) {
    return $res->code;
  } else {
    $o_nonotify = 1; # set no_notify as notification itself works using POST request which doesn't work
    my $content = decode_json($res->content);
    end_script(2, "Error issuing POST request on API: " . $res->status_line . " - " . $content->{full_error});
  }
}

sub delete_op5_api_url {
  my $url = shift;

  if ($o_pretend) {
    do_msg("info", "pretending op5 API DELETE request on URL: " . $url);
    return 500;
  }

  my $http_browser = prepare_http_request();
  my $req = HTTP::Request->new(DELETE => $url);

  my $res = $http_browser->request($req);

  if ($res->is_success) {
    return $res->code;
  } else {
    my $content = decode_json($res->content);
    end_script(2, "Error issuing DELETE request on API: " . $res->status_line . " - " . $content->{full_error});
  }
}

sub patch_op5_api_url {
  my $url = shift;
  my $json = shift;

  if ($o_pretend) {
    do_msg("info", "pretending op5 API PATCH request on URL: " . $url);
    do_msg("info", $json);
    return 500;
  }

  my $http_browser = prepare_http_request();

  my $req = HTTP::Request->new(PATCH => $url);
  $req->content_type('application/json');
  $req->content($json);

  my $res = $http_browser->request($req);
  if ($res->is_success) {
    return $res->code;
  } else {
    my $content = decode_json($res->content);
    end_script(2, "Error issuing PATCH request on API: " . $res->status_line . " - " . $content->{full_error});
  }
}

sub op5_api_create_contact {
  my $name = shift;
  my $realname = shift;
  my $email = shift;
  my $contactgroup = shift;

  my $host_notifications_enabled = JSON::PP::false;
  my $service_notifications_enabled = JSON::PP::false;
  my $host_notification_commands = "host-notify";
  my $service_notification_commands = "service-notify";

  my @host_notification_options = ["d","r"];
  my @service_notification_options = ["c","w","r"];

  if ($config->{contact_notifications_settings}) {
    if ($config->{contact_notifications_settings}->{host_notifications_enabled} and $config->{contact_notifications_settings}->{host_notifications_enabled} eq "true") {
      $host_notifications_enabled = JSON::PP::true;
    }

    if ($config->{contact_notifications_settings}->{service_notifications_enabled} and $config->{contact_notifications_settings}->{service_notifications_enabled} eq "true") {
      $service_notifications_enabled = JSON::PP::true;
    }

    if ($config->{contact_notifications_settings}->{host_notification_commands}) {
      $host_notification_commands = $config->{contact_notifications_settings}->{host_notification_commands};
    }

    if ($config->{contact_notifications_settings}->{service_notification_commands}) {
      $service_notification_commands = $config->{contact_notifications_settings}->{service_notification_commands};
    }


    if ($config->{contact_notifications_settings}->{host_notification_options}) {
      @host_notification_options = split(/;/, $config->{contact_notifications_settings}->{host_notification_options});
    }

    if ($config->{contact_notifications_settings}->{service_notification_options}) {
      @service_notification_options = split(/;/, $config->{contact_notifications_settings}->{service_notification_options});
    }
  }

  my %contact = (
    'contact_name' => $name,
    'alias' => $realname,
    'email' => $email,
    'register' => JSON::PP::true,
    'template' => 'default-contact',
    'contactgroups' => [ $contactgroup ],
    'host_notifications_enabled' => $host_notifications_enabled,
    'service_notifications_enabled' => $service_notifications_enabled,
    'host_notification_cmds' => $host_notification_commands,
    'service_notification_cmds' => $service_notification_commands,
    'host_notification_options' => @host_notification_options,
    'service_notification_options' => @service_notification_options,
    'can_submit_commands' => JSON::PP::true,
    'retain_status_information' => JSON::PP::true,
    '_GENERATED' => 'api-scripting',
    'retain_nonstatus_information' => JSON::PP::true
  );
  return post_op5_api_url( 'https://'.$config->{op5api}->{server}.'/api/config/contact', (encode_json( \%contact )) );
}

sub op5_api_create_service {
  my $host_object_name = shift;
  my $service_description = shift;

  my %service = (
    'host_name' => $host_object_name,
    'service_description' => $service_description,
    'check_command' => 'check_dummy',
    'check_command_args' => '2!"API script has not being run for at least 24 hours"!0',
    'template' => 'default-service',
    'register' => JSON::PP::true,
    'max_check_attempts' => 1,
    'check_interval' => 5,
    'retry_interval' => 1,
    'active_checks_enabled' => JSON::PP::false,
    'passive_checks_enabled' => JSON::PP::true,
    'check_period' => '24x7',
    'check_freshness' => JSON::PP::true,
    'freshness_threshold' => 86400
  );

  return post_op5_api_url( 'https://'.$config->{op5api}->{server}.'/api/config/service', (encode_json( \%service )) );
}

sub get_op5_all_hosts {
  my $url = 'https://'.$config->{op5api}->{server}.'/api/config/host';
  my $content = decode_json(get_op5_api_url($url));

  my @return;
  foreach (@$content) {
    push (@return, $_->{name});
  }
  return @return
}

sub get_op5_all_services {
  my $url = 'https://'.$config->{op5api}->{server}.'/api/config/service';
  my $content = decode_json(get_op5_api_url($url));

  my @return;
  foreach (@$content) {
    push (@return, $_->{name});
  }
  return @return
}

sub get_op5_all_contactgroups {
  my $url = 'https://'.$config->{op5api}->{server}.'/api/config/contactgroup';
  my $content = decode_json(get_op5_api_url($url));

  my @return;
  foreach (@$content) {
    push (@return, $_->{name});
  }
  return @return
}

sub get_op5_contactgroup_members {
  my $contactgroup = shift;
  my $url = 'https://' . $config->{op5api}->{server} . '/api/config/contactgroup/' . uri_escape($contactgroup);

  my $content = decode_json(get_op5_api_url($url));

  return @{$content->{members}}; 
}

sub check_monitor_host_has_service {
  my $host_object_name = shift;
  my $service_description = shift;
  my $return;

  my @services = get_op5_all_services();

  foreach (@services) {
    my ($h, $s) = split(';');
    if ($h eq $host_object_name) {
      if ($s eq $service_description) {
        $return = 1;
      }
    }
  }
  return $return;
}

sub check_monitor_has_contactgroup {
  my $contactgroup_name = shift;
  my $return;

  my @contactgroups = get_op5_all_contactgroups();

  foreach (@contactgroups) {
    if ($_ eq $contactgroup_name) {
      do_msg("info","found matching contact group name " . $_);
      $return = 1;
    }
  }
  return $return;
}

sub check_monitor_has_host {
  my $host_object_name = shift;
  my $return;

  my @hosts = get_op5_all_hosts();

  foreach (@hosts) {
    if ($_ eq $host_object_name) {
      do_msg("info","found matching host name " . $_);
      $return = 1;
    }
  }
  return $return;
}

sub check_monitor_has_contact {
  my $contact_name = shift;
  my $return;

  my $url = 'https://'.$config->{op5api}->{server}.'/api/config/contact/'.uri_escape($contact_name);
  my $http_browser = prepare_http_request();
  my $response = $http_browser->get($url);

  if ($response->is_success) {
    return 1;
  } else {
    return 0;
  }
}

sub sync_ldap_group_members_with_contactgroup {
  my $contactgroup = shift;
  my $contactgroup_members_ref = shift;
  my $ldap_group_members_ref = shift;

  my @contactgroup_members = sort(@$contactgroup_members_ref);
  my @ldap_group_members = sort(@$ldap_group_members_ref);

  if (@ldap_group_members ~~ @contactgroup_members) {

    do_msg("info", "no update of group members needed for $contactgroup");
    return;

  } else {

    do_msg("info", "updating group members for $contactgroup");
    my %element = (
      'members' => \@ldap_group_members
    );
    my $json = encode_json(\%element);
    my $url = 'https://' . $config->{op5api}->{server} . '/api/config/contactgroup/' . uri_escape($contactgroup);
    return patch_op5_api_url($url, $json);

  }
}

sub op5_api_delete_contacts_without_contactgroups {
  my $url = 'https://'.$config->{op5api}->{server}.'/api/config/contact';
  my $content = decode_json(get_op5_api_url($url));
  my $count = 0;

  my @return;
  foreach (@$content) {
    my $resource_url = $_->{resource};
    my $contact_details = decode_json(get_op5_api_url($resource_url));

    if ($contact_details->{_GENERATED} and $contact_details->{_GENERATED} eq 'api-scripting') {

      if (scalar(@{$contact_details->{contactgroups}}) == 0) {
        $count++;
        do_msg("info", "contact object " . $contact_details->{contact_name} . " has no contactgroups set, deleting");
        delete_op5_api_url($resource_url);
      }

    }
  }
  return $count;
}

sub op5_api_check_and_save {
  sleep 1;
  my $need_to_save = get_op5_api_url('https://' . $config->{op5api}->{server} . '/api/config/change');

  if (! $o_nosave) {
    if ($need_to_save && $need_to_save ne "[]") {
      do_msg("info", "saving the configuration to op5 Monitor API");
      post_op5_api_url('https://' . $config->{op5api}->{server} . '/api/config/change');
    }
  }
}

sub ldap_walk_through_group_members {
  my $ldap_group_members_ref = shift;
  my $group_dn = shift;
  my $group_shortname = shift;
  my $contactgroup_name = shift;
  my $ldap_memberkey = shift;

  foreach my $ldap_group_member (@{$ldap_group_members_ref}) {

    do_msg("info","found member $ldap_group_member in group $group_dn");

    # gather detailed information on this group member
    my $member_mesg;

    if ($authyml->{memberkey_is_dn} eq "1") { 

      $member_mesg = $ldap->search (
        base => $ldap_group_member,
        scope => 'base',
        filter => $authyml->{user_filter},
        sizelimit => 1,
        attrs => [ $authyml->{userkey}, $authyml->{userkey_realname}, $authyml->{userkey_email} ]
        );

    } else {

      $member_mesg = $ldap->search (
        base => $authyml->{user_base_dn},
        scope => 'subtree',
        filter => '(&(' . $authyml->{userkey} . '=' . $ldap_group_member . ')' .$authyml->{user_filter} . ')',
        sizelimit => 1,
        attrs => [ $authyml->{userkey}, $authyml->{userkey_realname}, $authyml->{userkey_email} ]
        );

    }

    # find out whether this member is a user or a group object
    $total_affected_ldap_users++;


    if ($member_mesg->code != 0) {
      end_script(2, "LDAP error during search for user $ldap_group_member");
    }

    if ($member_mesg->count == 0) {
      # seems not to be a user though
      my $nested_group_mesg;

      if ($authyml->{memberkey_is_dn} eq "1") {

        $nested_group_mesg = $ldap->search (
          base => $ldap_group_member,
          scope => 'base',
          filter => $authyml->{group_filter},
          attrs => [ $ldap_memberkey, $authyml->{groupkey} ]
          );

      } else {

        $nested_group_mesg = $ldap->search (
          base => $authyml->{group_base_dn},
          scope => 'subtree',
          filter => '(&(' . $authyml->{groupkey} . '=' . $ldap_group_member . ')' .$authyml->{group_filter} . ')',
          attrs => [ $ldap_memberkey, $authyml->{groupkey} ]
          );

      }

      if ($nested_group_mesg->code != 0) {
        end_script(2, "LDAP error during search for nested group");
      }

      my $nrofentries = $nested_group_mesg->count;
      if ($nrofentries == 0) {
        end_script(2, "LDAP error, member element $ldap_group_member doesn't seem to be a user nor a group");
      }

      do_msg("info", "$ldap_group_member is a nested group inside group $group_dn");
      my $nested_group_entry = $nested_group_mesg->entry;
      my $group_dn = $nested_group_entry->dn;
      my $ldap_group_members_ref; # reference to an array of members of the group

      if ($nested_group_entry->exists($ldap_memberkey)) {
        $ldap_group_members_ref = $nested_group_entry->get_value($ldap_memberkey, asref => 1);
        # recursively call myself
        my $group_shortname = $nested_group_entry->get_value($authyml->{groupkey});
        ldap_walk_through_group_members($ldap_group_members_ref, $group_dn, $group_shortname, $contactgroup_name, $ldap_memberkey);
      }

    } else {

      my $member_entry = $member_mesg->entry;
      my $ldap_group_member_shortname = $member_entry->get_value($authyml->{userkey});
      my $ldap_group_member_realname = $member_entry->get_value($authyml->{userkey_realname});
      my $ldap_group_member_email = $member_entry->get_value($authyml->{userkey_email});

      # check if user key is UPN format, and cut everything after the @ sign off in case it is
      if ($authyml->{userkey_is_upn} ne "false") {
        $ldap_group_member_shortname =~ s/@.*$//;
      }

      push(@ldap_group_members_shortnames, $ldap_group_member_shortname);

      ## create this user if not already existing
      if (! check_monitor_has_contact($ldap_group_member_shortname) ) {
        do_msg("info", "adding contact $ldap_group_member_shortname because of LDAP group $group_shortname");
        $added_contacts++;
        op5_api_create_contact(
          $ldap_group_member_shortname, 
          $ldap_group_member_realname, 
          $ldap_group_member_email, 
          $contactgroup_name
        );  
      }
    }
  }
}


#### MAIN ####
$authyml = load_authyml();

# open connection to syslog
openlog("op5-ldap-helper","pid","local3");

# check if the intention of the user is just to add the service check. In case it is, just do that and exit
if ($o_add_notification_service_check) {

  # first, check if the necessary configuration values are existing
  if ( !$config->{notify_service_check} or !$config->{notify_service_check}->{notify_service_check} or !$config->{notify_service_check}->{host_object_name} or !$config->{notify_service_check}->{service_check_description} ) {
  	print "Can only add service if all necessary configuration values are properly set\n";
  	exit;
  }

  print "Trying to add service check object with description \"" . $config->{notify_service_check}->{service_check_description} . "\" to host object \"". $config->{notify_service_check}->{host_object_name} ."\"...\n";

  # first, check if the host object exists
  if ( check_monitor_has_host( $config->{notify_service_check}->{host_object_name} ) ) {

    # check for existence of the service check at the host
    if ( check_monitor_host_has_service( $config->{notify_service_check}->{host_object_name}, $config->{notify_service_check}->{service_check_description} ) ) {
      print "A service check \"" . $config->{notify_service_check}->{service_check_description} . "\" is already existing on host \"" . $config->{notify_service_check}->{host_object_name} . "\"!\n";
      exit;
    } else {

      # add the service check
      op5_api_create_service($config->{notify_service_check}->{host_object_name}, $config->{notify_service_check}->{service_check_description});
      my $return = op5_api_check_and_save();
      print "...done (" . $return . ")\n";
      exit;

    }


  } else {
    print "A host object with name \"" . $config->{notify_service_check}->{host_object_name} . "\" does not exist in your Monitor configuration. Add it yourself or change config accordingly!\n";
    exit;
  }
}

# make connection to ldap
$ldap = ldap_con();
if (!$ldap) {
  end_script(2, "LDAP error, could not connect");
}
my $mesg = ldap_bind($ldap, $authyml->{bind_dn}, $authyml->{bind_secret});

my $ldap_memberkey = $authyml->{memberkey};
$ldap_memberkey =~ s/:.*$//;

my $ldap_group_base_dn;
if ($config->{ldap}->{custom_group_base_dn} 
    and $config->{ldap}->{custom_group_base_dn} ne "false" 
    and $config->{ldap}->{custom_group_base_dn} ne "") {
  $ldap_group_base_dn = $config->{ldap}->{custom_group_base_dn};
} else {
  $ldap_group_base_dn = $authyml->{group_base_dn};
}
chomp $ldap_group_base_dn;

my $ldap_group_search_scope;
if ($config->{ldap}->{custom_group_search_scope} 
    and $config->{ldap}->{custom_group_search_scope} ne "false" 
    and $config->{ldap}->{custom_group_search_scope} ne "") {
  $ldap_group_search_scope =  $config->{ldap}->{custom_group_search_scope};
} else {
  if ($authyml->{group_recursive} and $authyml->{group_recursive} ne "false" and $authyml->{group_recursive} ne "") {
    $ldap_group_search_scope = "sub";
  } else {
    $ldap_group_search_scope = "one";
  }
}

my $ldap_group_search_filter;
if ($config->{ldap}->{custom_group_filter} and $config->{ldap}->{custom_group_filter} ne "false" and $config->{ldap}->{custom_group_filter} ne "") {
  $ldap_group_search_filter = $config->{ldap}->{custom_group_filter};
} else {
  $ldap_group_search_filter = $authyml->{group_filter};
}

do_msg("info", "using LDAP base DN: " . $ldap_group_base_dn . ", scope: " . $ldap_group_search_scope . " for group search in LDAP");
do_msg("info", "filter for LDAP group search is: " . $ldap_group_search_filter);

$mesg = $ldap->search (
	base => $ldap_group_base_dn,
	scope => $ldap_group_search_scope,
	filter => $ldap_group_search_filter,
	attrs => [ $ldap_memberkey, $authyml->{groupkey} ]
	);


if ($mesg->code != 0) {
  end_script(2, "LDAP error during search for groups");
}

my $nrofentries = $mesg->count;
if ($nrofentries == 0) {
  end_script(2, "LDAP error, no groups below search base DN");
}

# walk through all groups and reformat the members
my $member_shortname;
my $usercomp_fromldap;


foreach my $entry ($mesg->entries) {
	my $group_dn = $entry->dn;

	# create group's shortname
  my $group_shortname = $entry->get_value($authyml->{groupkey});
  my $contactgroup_name = $config->{contactgroup_prefix} . " " . $group_shortname;

	do_msg("info","found group: $group_dn ($group_shortname)");

  if ( check_monitor_has_contactgroup($contactgroup_name) ) {
    $affected_ldap_groups++;

    my @contactgroup_members = get_op5_contactgroup_members($contactgroup_name);
    my $ldap_group_members_ref; # reference to an array of members of the group

    if ($entry->exists($ldap_memberkey)) {
      $ldap_group_members_ref = $entry->get_value($ldap_memberkey, asref => 1);
    }

    # walking through all the members of the found and matched LDAP group, whatever type of object
    # they are. It's now important to identify whether these are actual users or nested groups
    # and in case of nested groups walk through these

    ldap_walk_through_group_members($ldap_group_members_ref, $group_dn, $group_shortname, $contactgroup_name, $ldap_memberkey);
    $updated_contactgroups++ if sync_ldap_group_members_with_contactgroup($contactgroup_name, \@contactgroup_members, \@ldap_group_members_shortnames);

  }
}

#delete leftover contact objects
$deleted_contacts = op5_api_delete_contacts_without_contactgroups();

# check if save is necessary and save the configuration
op5_api_check_and_save();

$ldap->unbind;
$ldap->disconnect;

# some stats to finish gracefully
my $exitcode = 0;
if ($affected_ldap_groups == 0 or $total_affected_ldap_users == 0) {
  $exitcode = 1;
}
my $exitmsg = sprintf("%d affected LDAP groups, %d affected LDAP users in total - %d contacts added, %d contacts deleted, %d contactgroups updated", 
  $affected_ldap_groups, $total_affected_ldap_users, $added_contacts, $deleted_contacts, $updated_contactgroups);

$exitmsg .= sprintf(" | affected_ldap_groups=%d;;;; affected_total_users=%d;;;; contacts_added=%d;;;; contacts_deleted=%d;;;; contactgroups_updated=%d;;;;", 
  $affected_ldap_groups, $total_affected_ldap_users, $added_contacts, $deleted_contacts, $updated_contactgroups);

end_script($exitcode, $exitmsg);
